<?php

namespace Drupal\css_size_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\css_size_field\SizeUnit;

#[FieldWidget(
  id: "css_size_default",
  label: new TranslatableMarkup("Default"),
  field_types: ["css_size"]
)]
class CssSizeDefaultWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'default_unit' => '',
        'available_units' => [],
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['default_unit'] = [
      '#type' => 'select',
      '#title' => $this->t('Default unit'),
      '#options' => $this->getUnits(),
      '#default_value' => $this->getSetting('default_unit') ?: '',
    ];

    $element['available_units'] = [
      '#title' => $this->t('Allowed units'),
      '#description' => $this->t('Select the units to display, selecting none will display all.'),
      '#type' => 'select',
      '#multiple' => TRUE,
      '#size' => 10,
      '#options' => $this->getUnits(),
      '#default_value' => $this->getSetting('available_units') ?: [],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    if (!empty($this->getSetting('default_unit'))) {
      $summary[] = $this->t('Default unit: @unit', ['@unit' => $this->getSetting('default_unit')]);
    }
    $summary[] = $this->t('Available units: @units', ['@units' => empty($this->getSetting('available_units')) ? $this->t('All') : implode(', ', $this->getSetting('available_units'))]);

    return $summary;
  }

  /**
   * Gets the available units for the current field.
   *
   * @return array
   *   The unit labels, keyed by unit.
   */
  protected function getUnits() {
    return SizeUnit::getLabels();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    if ($items[$delta]->isEmpty()) {
      $items[$delta]->number = NULL;
      $items[$delta]->unit = $this->getSetting('default_unit');
    }

    $element = [
      '#type' => 'css_size',
      '#default_value' => $items[$delta]->getValue(),
    ] + $element;
    if (array_filter($this->getSetting('available_units'))) {
      $element['#available_units'] = $this->getSetting('available_units');
    }

    return $element;
  }

}
