<?php

namespace Drupal\css_size_field;

/**
 * Provides size units.
 */
final class SizeUnit implements UnitInterface {

  const CENTIMETER = 'cm';
  const MILLIMETER = 'mm';
  const QUARTER_MILLIMETER = 'Q';
  const INCH = 'in';
  const PICAS = 'pc';
  const POINT = 'pt';
  const PIXEL = 'px';
  const RELATIVE_ELEMENT = 'em';
  const RELATIVE_ROOT_ELEMENT = 'rem';
  const VIEWPORT_WIDTH = 'vw';
  const VIEWPORT_HEIGHT = 'vh';
  const PERCENT = '%';

  /**
   * {@inheritdoc}
   */
  public static function getLabels() {
    return [
      self::CENTIMETER => t('cm'),
      self::MILLIMETER => t('mm'),
      self::QUARTER_MILLIMETER => t('Q'),
      self::INCH => t('in', [], ['context' => 'unit']),
      self::PICAS => t('pc'),
      self::POINT => t('pt'),
      self::PIXEL => t('px'),
      self::RELATIVE_ELEMENT => t('em'),
      self::RELATIVE_ROOT_ELEMENT => t('rem'),
      self::VIEWPORT_WIDTH => t('vw'),
      self::VIEWPORT_HEIGHT => t('vh'),
      self::PERCENT => t('%'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function assertExists($unit) {
    $allowed_units = [
      self::CENTIMETER,
      self::MILLIMETER,
      self::QUARTER_MILLIMETER,
      self::INCH,
      self::PICAS,
      self::POINT,
      self::PIXEL,
      self::RELATIVE_ELEMENT,
      self::RELATIVE_ROOT_ELEMENT,
      self::VIEWPORT_WIDTH,
      self::VIEWPORT_HEIGHT,
      self::PERCENT,
    ];
    if (!in_array($unit, $allowed_units)) {
      throw new \InvalidArgumentException(sprintf('Invalid weight unit "%s" provided.', $unit));
    }
  }

  /**
   * Returns a list of all defined size unit constants.
   *
   * @return array
   *   An array of all size units.
   */
  public static function getAllUnits() {
    return [
      self::CENTIMETER,
      self::MILLIMETER,
      self::QUARTER_MILLIMETER,
      self::INCH,
      self::PICAS,
      self::POINT,
      self::PIXEL,
      self::RELATIVE_ELEMENT,
      self::RELATIVE_ROOT_ELEMENT,
      self::VIEWPORT_WIDTH,
      self::VIEWPORT_HEIGHT,
      self::PERCENT,
    ];
  }
}
